/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/


#if !defined(__DELTAPLANE_H__)
#define __DELTAPLANE_H__

#include "object.h"
#include "physic.h"

// DeltaPlane class
class DeltaPlane : public Object{
  public:
    DeltaPlane(const Point& position, double width, double span, int resolution, double mass, double massperson);
    virtual ~DeltaPlane();

    void                 registerInSimulator(PhysicSimulatorVerlet* sim);

    // Permet de controler le pilote
    void                 controlPosition(double dx, double dy);

    Particule*           getPerson() { return &person; }
    Point&               getPosPerson() { return person.pos;}

    // Affiche le deltaplane et son pilote
    virtual void         renderGL() const;

    // Dplace le deltaplane
    void                 moveTo(const Point& dst);

  protected:
    // Calcule les vecteurs normaux
    void                 computeNormals();

    int                  resolution;    // Rsolution des cts (en nombre de stick ou particule-1)
    Particule*           partsToile;    // Particules de la toile (triangle principal)
    int                  nbPartsToile;  // Nb de particules dans la toile

    // Particules supplmentaires pour la rigidit
    Particule            hautToile;
    Particule            centre;
    Particule            basToile;

    // Control of the person
    int*                   controlSticks;   // sticks indexes
    Point*                 controlPoints;   // original positions
    Particule              person;          // particule that represent the person
    Point                  controlCible;    // Where we want to put the person...
    PhysicSimulatorVerlet* simulator;

    // For display...
    Vector*                normalsToile;  // Vecteurs normaux
};


// Essai d'une nouvelle structure de deltaplane
class DeltaPlane2 : public Object{
  public:
    DeltaPlane2(const Point& position, double width, double span, int resolution, double mass, double massperson);
    virtual ~DeltaPlane2();

    void                 registerInSimulator(PhysicSimulatorVerlet* sim);

    void                 controlPosition(double dx, double dy);

    Particule*           getPerson() { return &person; }
    Point&               getPosPerson() { return person.pos;}

    virtual void         renderGL() const;

    void                 stabilize();
    void                 moveTo(const Point& dst);

  protected:
    void                 computeNormals();

    int                  resolution;
    Particule*           partsToile;
    Vector*              normalsToile;
    int                  nbPartsToile;

    Particule            hautToile;
    Particule            centre;
    Particule            basToile;

    // Control of the person
    int*                   controlSticks;   // sticks indexes
    Point*                 controlPoints;   // original positions
    Particule              person;          // particule that represent the person
    Point                  controlCible;    // Where we want to put the person...
    PhysicSimulatorVerlet* simulator;
};



#endif

