/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://siela.dmi.usherb.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include "geometrie.h"
#include <math.h>

Point::Point(double x_, double y_, double z_)
 : x(x_), y(y_), z(z_)
{
}

Point Point::operator+(const Vector& v) const{
   Point p = *this;
   return p+=v;
}

Point& Point::operator+=(const Vector& v){
   x += v.x;
   y += v.y;
   z += v.z;
   return *this;
}

Point Point::operator-(const Vector& v) const{
   Point p = *this;
   return p-=v;
}

Point& Point::operator-=(const Vector& v){
   x -= v.x;
   y -= v.y;
   z -= v.z;
   return *this;
}

Vector Point::operator - (const Point& p) const{
   return Vector(x-p.x, y-p.y, z-p.z);
}

Point  Point::operator *  (const double& v) const{
   return Point(x*v, y*v, z*v);
}

Point& Point::operator *= (const double& v){
   x *= v;
   y *= v;
   z *= v;
   return *this;
}

double Point::distance(const Point& p) const{
   double d, dc;
   dc = x - p.x;
   d = dc*dc;
   dc = y - p.y;
   d += dc*dc;
   dc = z - p.z;
   d += dc*dc;
   return sqrt(d);
}

double Point::distance2(const Point& p) const{
   double d, dc;
   dc = x - p.x;
   d = dc*dc;
   dc = y - p.y;
   d += dc*dc;
   dc = z - p.z;
   d += dc*dc;
   return d;
}



Vector::Vector(double x_, double y_, double z_)
 : x(x_), y(y_), z(z_)
 {
 }

 Vector::Vector(const Point& p1, const Point& p2){
     x = p2.x - p1.x;
     y = p2.y - p1.y;
     z = p2.z - p1.z;
 }

 Vector Vector::operator + (const Vector& v) const{
     Vector r(*this);
     return r+=v;
 }

 Vector& Vector::operator += (const Vector& v){
     x += v.x;
     y += v.y;
     z += v.z;
     return *this;
}

Vector Vector::operator -  () const {
    return Vector(-x, -y, -z);
}

 Vector Vector::operator - (const Vector& v) const{
     Vector r(*this);
     return r-=v;
 }

 Vector& Vector::operator -= (const Vector& v){
     x -= v.x;
     y -= v.y;
     z -= v.z;
     return *this;
}

Vector Vector::operator ^ (const Vector& v) const{
     return Vector(y*v.z - v.y*z,
                   z*v.x - v.z*x,
                   x*v.y - v.x*y);
}

Vector& Vector::operator ^= (const Vector& v) {
     *this = *this ^ v;
     return *this;
}

Vector Vector::operator *  (const double& v) const{
      return Vector(x*v, y*v, z*v);
}

Vector Vector::operator *= (const double& v) {
      x *= v;
      y *= v;
      z *= v;
      return *this;
}

double Vector::operator *  (const Vector& v) const{
     return x*v.x + y*v.y + z*v.z;
}

Vector Vector::operator /  (const double& v) const{
     return Vector(x/v, y/v, z/v);
}

Vector Vector::operator /= (const double& v){
      x /= v;
      y /= v;
      z /= v;
      return *this;
}

double Vector::norm() const{
     return sqrt(x*x + y*y + z*z);
}

void  Vector::normalize(){
    double n = norm();
    x /= n;
    y /= n;
    z /= n;
}

double Vector::norm2() const{
     return x*x + y*y + z*z;
}

Vector Vector::projectOn(const Vector& u) const{
     return  u * (*this * u / u.norm2()) ;
}

std::ostream& operator << (std::ostream& out, const Point& p){
   out << "(" << p.x << ", " << p.y << ", " << p.z << ")";
   return out;
}

std::ostream& operator << (std::ostream& out, const Vector& v){
   out << "(" << v.x << ", " << v.y << ", " << v.z << ")";
   return out;
}
