/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include <iostream>
#include <assert.h>
#include <GL/glut.h>
#include "kite.h"
#include "displayglut.h"
#include "geometrie.h"


Kite::Kite(const Point& ropeattach, const Point& position0, const Vector& width, const Vector& height,
           double masskite, int resolution_, int roperes_)
  : resolution(resolution_), resRope(roperes_)
{
   // Create main Triangle
   assert(resolution >= 1);
   nbPartsKiteMain = (resolution_ + 1) * (resolution_ + 2) / 2;

   keel1 = 2;  // rang de dbut du keel
   keel2 = resolution / 2; // rang de fin du keel
   resKeel = keel2 - keel1;
   assert(resKeel >= 1 && keel2<=resolution/2);
   nbPartsKiteKeel = (resKeel + 0) * (resKeel + 1) / 2; // La dernire ligne du keel est commune avec le main triangle...
   double massPart = masskite / (nbPartsKiteMain + nbPartsKiteKeel);

   partsKite = new Particule[nbPartsKiteMain];
   normalsKite = new Vector[nbPartsKiteMain];
   int n=0;
   for(int i=0;i<=resolution;i++)
      for(int j=0;j<=resolution-i;j++){
         double a = i + j;
         double b = j - i;
         Vector v = width * (0.5 * b/resolution) + height * (a/resolution);
         partsKite[n].pos = position0 + v;
         std::cout << "partsKite[" << n << "]=" << partsKite[n].pos << std::endl;
         partsKite[n].mass = massPart;
         n++;
      }
   assert(n==nbPartsKiteMain); // To be sure that we have the correct number of particules


   // Create Keel
   partsKeel = new Particule[nbPartsKiteKeel];
   Vector elevation = height ^ width;
   std::cout << "elevation=" << elevation << std::endl;
   std::cout << "width=" << width << std::endl;
   std::cout << "height=" << height << std::endl;
   elevation *= width.norm() / (50 * elevation.norm());
   Point keelSommet1 = position0 + (height*0.6 + elevation);
   std::cout << "position0=" << position0 << "  keelSommet1=" << keelSommet1 << std::endl;
   Point keelSommet2, keelSommet3;

   n=0;
   for(int i=0;i<=keel2;i++) {
      if(i==keel1)
         keelSommet2 = partsKite[n].pos;
      if(i==keel2)
         keelSommet3 = partsKite[n].pos;
      n += (resolution-i+2);
   }

   n = 0;
   for(int i=0;i<resKeel;i++) { // tage de bas en haut
      Point p1 = keelSommet1 + (keelSommet2-keelSommet1) * ((double)i / resKeel);
      Point p2 = keelSommet1 + (keelSommet3-keelSommet1) * ((double)i / resKeel);
      Vector v12 = p2-p1;
      for(int j=0;j<=i;j++) {   // dans le sens de height
         //double a = (2*j+resKeel-i) / ((resKeel) * 2.0);
         //double b = 1.0 - (double)i / resKeel;
         //partsKeel[n].pos = position0 + height * a + elevation * b;
         partsKeel[n].pos = i==0 ? p1 : p1 + v12 * ((double)j / i);
         std::cout << "partsKeel[" << n << "].pos=" << partsKeel[n].pos << std::endl;
         partsKeel[n].mass = massPart / 4;
         n++;
      }
   }
   assert(n==nbPartsKiteKeel); // To be sure that we have the correct number of particules

   haut.pos = position0 + height*0.6 + elevation * 17;
   haut.mass = massPart;
   bas.pos = position0 + height*0.6 - elevation * 17;
   bas.mass = massPart; // * 10;

   // Create rope
   partsRope = new Particule[resRope+1];
   Vector initRope = keelSommet1 - ropeattach;
   for(int i=0;i<=resRope;i++){
      partsRope[i].pos = ropeattach + initRope * ((double) i / resRope);
      std::cout << "partsRope[i].pos=" << partsRope[i].pos << std::endl;
      partsRope[i].mass = massPart / 1.0;
   }
}

Kite::~ Kite(){
   delete[] partsKite;
   delete[] partsKeel;
   delete[] partsRope;
   delete[] normalsKite;
}

void Kite::registerInSimulator(PhysicSimulatorVerlet* sim){
   //Register Particules
   for(int i=0;i<nbPartsKiteMain;i++)
      sim->addParticule(partsKite + i);
   for(int i=0;i<nbPartsKiteKeel;i++)
      sim->addParticule(partsKeel+i);
   for(int i=0;i<=resRope;i++)
      sim->addParticule(partsRope+i);

   sim->addParticule(&haut);
   sim->addParticule(&bas);

   sim->addStick(Stick(&bas, &haut));

   // Sticks + surface
   //Main triangle
   for(int i=0, n=0;i<resolution;i++){
      for(int j=0;j<resolution-i;j++){
         Particule* a = partsKite + n;
         Particule* b = partsKite + n + 1;
         Particule* c = partsKite + n + (resolution-i + 1);

         sim->addStick(Stick(a, b));
         sim->addStick(Stick(a, c));
         sim->addStick(Stick(b, c));

         sim->addSurface(SurfaceTriangle(a, b, c));

         if(j<resolution-i-1){
            Particule* d = partsKite + n + (resolution-i + 2);
            sim->addSurface(SurfaceTriangle(d, c, b));
         }
         n++;
      }
      n++;
   }

   // Rigidity of sides and central
   for(int i=0, b=0, c=0;i<=resolution;i++){
      sim->addStick(Stick(partsKite + i, &haut));
      sim->addStick(Stick(partsKite + i, &bas));

      if(i>0){
         sim->addStick(Stick(partsKite + c, &haut));
         sim->addStick(Stick(partsKite + c, &bas));
         //Stick temp(partsKite + c, partsKite + i);
         //temp.setDistance(0.85 * temp.getDistance());
         //sim->addStick(temp); // en largeur

         if(i<=resolution/2){
            sim->addStick(Stick(partsKite + b, &haut));
            sim->addStick(Stick(partsKite + b, &bas));
            sim->addStick(Stick(partsKite + b, partsKeel + 0)); // centre avec bas keel
         }
         if(i==resolution/2){
             assert(b<nbPartsKiteMain);
             sim->addStick(Stick(partsKite + i, partsKite + c)); // en largeur
             sim->addStick(Stick(partsKite + i, partsKite + b));
             sim->addStick(Stick(partsKite + c, partsKite + b));

             //sim->addStick(Stick(partsKeel, partsKite + b));
             //sim->addStick(Stick(partsKeel, partsKite));
         }
      }
      b += resolution - i + 2;
      c += resolution - i + 1;
   }

   // Keel
   std::cerr << "Register Keel.... nbPartKeel=" << nbPartsKiteKeel << std::endl;
   for(int i=0, n=0;i<resKeel-1;i++){
      for(int j=0;j<=i;j++){
         Particule* a = partsKeel + n;
         Particule* b = partsKeel + n + (i+1);
         Particule* c = partsKeel + n + (i+2);
         std::cerr << "n=" << n << "  i=" << i << std::endl;
         sim->addStick(Stick(a, b));
         sim->addStick(Stick(a, c));
         sim->addStick(Stick(b, c));

         sim->addSurface(SurfaceTriangle(a, b, c));
         if(j<i){
            Particule* d = partsKeel + n + 1;
            sim->addSurface(SurfaceTriangle(a, c, d));
         }
         n++;
      }
   }

   //Jonction du keel avec le triangle principale
   for(int i=0, n1=0;i<keel2;i++){
      if(i>=keel1){
         Particule* a = partsKite + n1;
         //std::cout << "keel #" << (nbPartsKiteKeel - resKeel + i - keel1) << std::endl;
         Particule* b = &partsKeel[nbPartsKiteKeel - resKeel + i - keel1];
         Particule* c = partsKite + n1 + (resolution-i+2);

         sim->addStick(Stick(a, b));
         sim->addStick(Stick(a, c));
         sim->addStick(Stick(b, c));
         sim->addSurface(SurfaceTriangle(a, b, c));

         if(i<keel2-1){
            Particule* d = &partsKeel[nbPartsKiteKeel - resKeel + i - keel1 + 1];
            sim->addSurface(SurfaceTriangle(b, c, d));
         }
      }
      n1 += (resolution-i+2);
   }


   // Rope
   for(int i=0;i<resRope;i++)
      sim->addStick(Stick(partsRope+i, partsRope+i+1));
   sim->addStick(Stick(partsRope+resRope, partsKeel+0));
   sim->addNail(Nail(partsRope+0));
}

void Kite::computeNormals(){

   for(int i=0;i<nbPartsKiteMain;i++)
      normalsKite[i].setZero();

  for(int i=0, a=0;i<resolution;i++){
    for(int j=0;j<resolution-i;j++){
       int c = a+1;
       int b = a+resolution-i+1;
       if(j<resolution-i-1){
         int d = b+1;
         Vector v1 = partsKite[b].pos - partsKite[a].pos;
         Vector v2 = partsKite[d].pos - partsKite[b].pos;
         Vector vn = v1 ^ v2;
         vn.normalize();
         normalsKite[a] += vn;
         normalsKite[b] += vn;
         normalsKite[d] += vn;

         v1 = partsKite[d].pos - partsKite[a].pos;
         v2 = partsKite[c].pos - partsKite[d].pos;
         vn = v1 ^ v2;
         vn.normalize();
         normalsKite[a] += vn;
         normalsKite[c] += vn;
         normalsKite[d] += vn;
       }else{
         Vector v1 = partsKite[b].pos - partsKite[a].pos;
         Vector v2 = partsKite[c].pos - partsKite[b].pos;
         Vector vn = v1 ^ v2;
         vn.normalize();
         normalsKite[a] += vn;
         normalsKite[b] += vn;
         normalsKite[c] += vn;
       }
       a++;
    }
    a++;
  }

   for(int i=0;i<nbPartsKiteMain;i++){
      normalsKite[i].normalize();
      //normalsToile[i] *= -1;
   }

}


void Kite::renderGL() const{
  ((Kite*)this)->computeNormals();
  glPushMatrix();

  glColor3f(0.8f, 0.4f, 0.3f);

  Point* p;
  Vector* v;

  // Draw main triangle
/*
  glNormal3f(0.0f, 0.0f, -1.0f);
  for(int i=0, n=0;i<=resolution;i++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int j=0;j<=resolution-i;j++){
         // Point 1
         p = &partsKite[n].pos;
         glVertex3d(p->x, p->y, p->z);

         if(j<resolution-i){
             p = &partsKite[n + resolution-i+1].pos;
             glVertex3d(p->x, p->y, p->z);
         }
         n++;
      }
      glEnd();
  }
*/
  glBegin(GL_TRIANGLES);
  for(int i=0, a=0;i<resolution;i++){
    for(int j=0;j<resolution-i;j++){
       int c = a+1;
       int b = a+resolution-i+1;
       if(j<resolution-i-1){
         int d = b+1;
         v = normalsKite + a;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[a].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + b;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[b].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + d;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[d].pos;
          glVertex3d(p->x, p->y, p->z);

         v = normalsKite + a;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[a].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + d;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[d].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + c;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[c].pos;
          glVertex3d(p->x, p->y, p->z);
       }else{
         v = normalsKite + a;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[a].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + b;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[b].pos;
          glVertex3d(p->x, p->y, p->z);
         v = normalsKite + c;
          glNormal3d(v->x, v->y, v->z);
          p = &partsKite[c].pos;
          glVertex3d(p->x, p->y, p->z);
       }
       a++;
    }
    a++;
  }
  glEnd();

  // Draw Keel
  glColor3f(0.8f, 0.9f, 0.4f);
   for(int i=0, n=0;i<resKeel-1;i++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int j=0;j<=i;j++){
         p = &partsKeel[n + i + 1].pos;
         glVertex3d(p->x, p->y, p->z);

         p = &partsKeel[n].pos;
         glVertex3d(p->x, p->y, p->z);
         n++;
      }
      p = &partsKeel[n + i + 1].pos;
      glVertex3d(p->x, p->y, p->z);
      glEnd();
   }
   //Dernier tage du Keel
   glBegin(GL_TRIANGLE_STRIP);
   for(int i=0, n1=0;i<=keel2;i++){
      if(i>=keel1){
         p = &partsKite[n1].pos;
         glVertex3d(p->x, p->y, p->z);

         if(i<keel2){
            p = &partsKeel[nbPartsKiteKeel - resKeel + i - keel1].pos;
            glVertex3d(p->x, p->y, p->z);
         }
      }
      n1 += (resolution-i+2);
   }
   glEnd();

   // Draw rope
   glColor3f(1, 1, 1);
   glBegin(GL_LINE_STRIP);
   for(int i=0;i<=resRope;i++){
      p = &partsRope[i].pos;
      glVertex3d(p->x, p->y, p->z);
   }
   p = &partsKeel[0].pos;
   //std::cout << "partsKeel[0].pos=" << partsKeel[0].pos << std::endl;
   glVertex3d(p->x, p->y, p->z);
   glEnd();

/*
   // Draw haut et bas
   glColor3f(0.2, 0.2, 0.8);
   glBegin(GL_LINES);
   const Point* ph = &haut.pos;
   const Point* pb = &bas.pos;
   for(int i=0, b=0, c=0;i<=resolution;i++){
      Point* p = &partsKite[i].pos;
      glVertex3d(p->x, p->y, p->z);
      glVertex3d(ph->x, ph->y, ph->z);
      glVertex3d(p->x, p->y, p->z);
      glVertex3d(pb->x, pb->y, pb->z);

      if(i>0){
         p = &partsKite[c].pos;
         glVertex3d(p->x, p->y, p->z);
         glVertex3d(ph->x, ph->y, ph->z);
         glVertex3d(p->x, p->y, p->z);
         glVertex3d(pb->x, pb->y, pb->z);

         if(i<=resolution/2){
            p = &partsKite[b].pos;
            glVertex3d(p->x, p->y, p->z);
            glVertex3d(ph->x, ph->y, ph->z);
            glVertex3d(p->x, p->y, p->z);
            glVertex3d(pb->x, pb->y, pb->z);
         }
      }
      b += resolution - i + 2;
      c += resolution - i + 1;
   }
   glEnd();
*/
  glPopMatrix();
}

