/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include <iostream>
#include <assert.h>
#include <GL/glut.h>
#include "kite2.h"
#include "displayglut.h"
#include "geometrie.h"


Kite2::Kite2(const Point& ropeattach, const Point& position0, const Vector& width, const Vector& height,
           double masskite, int resolution_, int roperes_, int keel1_, int keel2_,
           double fx, double fy)
  : resolution(resolution_), resRope(roperes_)
{
   // Create main Triangle
   assert(resolution >= 1);
   nbPartsKiteMain = (resolution_ + 1) * (resolution_ + 2) / 2;

   //keel1 = 2;  // rang de dbut du keel
   keel1 = keel1_;
   //keel2 = resolution / 2; // rang de fin du keel
   keel2 = resolution/2 - keel2_;

   resKeel = keel2 - keel1;
   assert(resKeel >= 1 && keel2<=resolution/2);
   nbPartsKiteKeel = (resKeel + 0) * (resKeel + 1) / 2; // La dernire ligne du keel est commune avec le main triangle...
   double massPart = 0.0025; //masskite / (nbPartsKiteMain + nbPartsKiteKeel);

   partsKite = new Particule[nbPartsKiteMain];
   int n=0;
   for(int i=0;i<=resolution;i++)
      for(int j=0;j<=resolution-i;j++){
         double a = i + j;
         double b = j - i;
         Vector v = width * (0.5 * b/resolution) + height * (a/resolution);
         partsKite[n].pos = position0 + v;
         //std::cout << "partsKite[" << n << "=" << partsKite[n].pos << std::endl;
         partsKite[n].mass = massPart;
         n++;
      }
   assert(n==nbPartsKiteMain); // To be sure that we have the correct number of particules


   // Create Keel
   Vector elevation =  width ^ height;
   std::cout << "elevation=" << elevation << std::endl;
   elevation *= height.norm() / (1.0 * elevation.norm());
   partsKeel = new Particule[nbPartsKiteKeel];
   Vector keel_dx = height * fx;
   Vector keel_dy = elevation * fy;
   Vector x1 = (height * (resKeel*2.0 / resolution) - keel_dx) / resKeel;
   Vector x2 = keel_dx / resKeel;
   Vector y1 = keel_dy / resKeel;
   //Point xoffset = position0 + height * ((keel1 + resKeel)*2.0 / resolution) - keel_dx - keel_dy;
   Point xoffset = position0 + height * ((keel1)*2.0 / resolution) + keel_dx - keel_dy;
   n = 0;
   for (int y = 0; y <= resKeel - 1; y++) {
      for (int x = 0; x <= resKeel - y - 1; x++) {
          Vector posx = - (x2*x - x1*y);
          Vector posy = y1*(x + y);
          partsKeel[n].pos = xoffset + posx + posy;
          partsKeel[n].mass = massPart;
          //std::cout << "relpos[" << n << "] = " << (posx+posy) << std::endl;
          //std::cout << "partkeel[" << n << "].pos = " << partsKeel[n].pos << std::endl;
          n++;
      }
      //std::cout << "---" << std::endl;
    }
   Point keelSommet1 = xoffset;

   /*
   //std::cout << "elevation=" << elevation << std::endl;
   //std::cout << "width=" << width << std::endl;
   //std::cout << "height=" << height << std::endl;
   Point keelSommet1 = position0 + (height*0.61 + elevation*0.10);
   //std::cout << "position0=" << position0 << "  keelSommet1=" << keelSommet1 << std::endl;
   Point keelSommet2, keelSommet3;

   for(int i=0, n=0;i<=keel2;i++) {
      if(i==keel1)
         keelSommet2 = partsKite[n].pos;
      if(i==keel2)
         keelSommet3 = partsKite[n].pos;
      n += (resolution-i+2);
   }

   n = 0;
   for(int i=0;i<resKeel;i++) { // tage de bas en haut
      Point p1 = keelSommet1 + (keelSommet2-keelSommet1) * ((double)i / resKeel);
      Point p2 = keelSommet1 + (keelSommet3-keelSommet1) * ((double)i / resKeel);
      Vector v12 = p2-p1;
      for(int j=0;j<=i;j++) {   // dans le sens de height
         //double a = (2*j+resKeel-i) / ((resKeel) * 2.0);
         //double b = 1.0 - (double)i / resKeel;
         //partsKeel[n].pos = position0 + height * a + elevation * b;
         partsKeel[n].pos = i==0 ? p1 : p1 + v12 * ((double)j / i);
         //std::cout << "partsKeel[" << n << "].pos=" << partsKeel[n].pos << std::endl;
         partsKeel[n].mass = massPart;
         n++;
      }
   }*/
   std::cout << "nbpartkeel = " << nbPartsKiteKeel << std::endl;
   assert(n==nbPartsKiteKeel); // To be sure that we have the correct number of particules

   haut.pos = position0 + height*0.5 + elevation * 1;
   haut.mass = massPart;
   bas.pos = position0 + height*0.5 - elevation * 1;
   bas.mass = massPart ; // * 10;

   // Create rope
   partsRope = new Particule[resRope];
   Vector initRope = keelSommet1 - ropeattach;
   for(int i=0;i<resRope;i++){
      partsRope[i].pos = ropeattach + initRope * ((double) i / resRope);
      //std::cout << "partsRope[i].pos=" << partsRope[i].pos << std::endl;
      partsRope[i].mass = massPart; //massPart / 10.0;
   }
}

Kite2::~ Kite2(){
   delete[] partsKite;
   delete[] partsKeel;
   delete[] partsRope;
}

void Kite2::registerInSimulator(PhysicSimulatorVerlet* sim){
   //Register Particules
   for(int i=0;i<nbPartsKiteMain;i++)
      sim->addParticule(partsKite + i);
   for(int i=0;i<nbPartsKiteKeel;i++)
      sim->addParticule(partsKeel+i);
   for(int i=0;i<resRope;i++)
      sim->addParticule(partsRope+i);

   sim->addParticule(&haut);
   sim->addParticule(&bas);

   sim->addStick(Stick(&bas, &haut));

   // Sticks + surface
   //Main triangle
   for(int i=0, n=0;i<resolution;i++){
      for(int j=0;j<resolution-i;j++){
         Particule* a = partsKite + n;
         Particule* b = partsKite + n + 1;
         Particule* c = partsKite + n + (resolution-i + 1);

         sim->addStick(Stick(a, b));
         sim->addStick(Stick(a, c));
         sim->addStick(Stick(b, c));

         if(j<resolution-i-1){
            Particule* d = partsKite + n + (resolution-i + 2);
            sim->addStick(Stick(a, d));
            sim->addSurface(SurfaceTriangle(a, b, d));
            sim->addSurface(SurfaceTriangle(a, d, c));
         }else{
            sim->addSurface(SurfaceTriangle(a, b, c));
         }
         n++;
      }
      n++;
   }

   // Rigidity of sides and central
   for(int i=0, b=0, c=0;i<=resolution;i++){
      sim->addStick(Stick(partsKite + i, &haut));
      sim->addStick(Stick(partsKite + i, &bas));

      if(i>0){
         sim->addStick(Stick(partsKite + c, &haut));
         sim->addStick(Stick(partsKite + c, &bas));
         //Stick temp(partsKite + c, partsKite + i);
         //temp.setDistance(0.85 * temp.getDistance());
         //sim->addStick(temp); // en largeur

         if(i<=resolution/2){
            sim->addStick(Stick(partsKite + b, &haut));
            sim->addStick(Stick(partsKite + b, &bas));
         }
         if(i==resolution/2){
             assert(b<nbPartsKiteMain);
             sim->addStick(Stick(partsKite + i, partsKite + c)); // en largeur
             sim->addStick(Stick(partsKite + i, partsKite + b));
             sim->addStick(Stick(partsKite + c, partsKite + b));

             //sim->addStick(Stick(partsKeel, partsKite + b));
             //sim->addStick(Stick(partsKeel, partsKite));
         }
      }
      b += resolution - i + 2;
      c += resolution - i + 1;
   }

   // Keel
   int centre = 0;
   for(int i=0;i<keel1;i++)
     centre += (resolution-i) + 2;
   for(int i=0, n=0;i<resKeel;i++){
      for(int j=0;j<resKeel-i;j++){
          Particule* a = partsKeel+n;
          Particule* b = partsKeel+n+1;
          Particule* c = partsKeel+n+(resKeel-i);
          if(j<resKeel-i-1){
             sim->addStick(Stick(a, b));
             sim->addStick(Stick(a, c));
             sim->addStick(Stick(b, c));
          }
          Particule* d = partsKeel+n+(resKeel-i)+1;
          if(j<resKeel-i-2){ // vers 2 niveaux suprieurs
             sim->addStick(Stick(a, d));
             sim->addSurface(SurfaceTriangle(a, d, b));
             sim->addSurface(SurfaceTriangle(a, c, d));
          }
          Particule* e = partsKite+centre;
          Particule* f = partsKite+centre + resolution-i-keel1+2;
          if(j==resKeel-i-2){ // longue jonction
             sim->addStick(Stick(partsKeel+n, f));
             sim->addSurface(SurfaceTriangle(a, f, b));
             sim->addSurface(SurfaceTriangle(a, c, f));
          }
          if(j==resKeel-i-1){ // jonction
             sim->addStick(Stick(a, e));
             sim->addStick(Stick(a,f));
             sim->addSurface(SurfaceTriangle(a, f, e));
          }
          n++;
      }
      centre += (resolution-i-keel1+2);
  }

   // Rope
   for(int i=0;i<resRope-1;i++){
      //for(int j=0;j<15;j++)
      sim->addStick(Stick(partsRope+i, partsRope+i+1));
   }
  //for(int j=0;j<15;j++)
      sim->addStick(Stick(partsRope+resRope-1, partsKeel+0));
      //sim->addStick(Stick(partsRope+resRope-1, partsKeel+1));
      //sim->addStick(Stick(partsRope+resRope-1, partsKeel+resKeel));
   sim->addNail(Nail(partsRope+0));
}

void Kite2::renderGL() const{
  glPushMatrix();

  glColor3f(0.8, 0.4, 0.3);
  glNormal3f(0.0, 0.0, -1.0);

  Point* p;

  // Draw main triangle
   for(int i=0, n=0;i<=resolution;i++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int j=0;j<=resolution-i;j++){
         // Point 1
         p = &partsKite[n].pos;
         glVertex3d(p->x, p->y, p->z);

         if(j<resolution-i){
             p = &partsKite[n + resolution-i+1].pos;
             glVertex3d(p->x, p->y, p->z);
         }
         n++;
      }
      glEnd();
  }

  // Draw Keel
  glColor3f(0.8, 0.9, 0.4);
   for(int i=0, n=0;i<resKeel-1;i++){
      glBegin(GL_TRIANGLE_STRIP);
      for(int j=0;j<=i;j++){
         p = &partsKeel[n + i + 1].pos;
         glVertex3d(p->x, p->y, p->z);

         p = &partsKeel[n].pos;
         glVertex3d(p->x, p->y, p->z);
         n++;
      }
      p = &partsKeel[n + i + 1].pos;
      glVertex3d(p->x, p->y, p->z);
      glEnd();
   }
   //Dernier tage du Keel
   glBegin(GL_TRIANGLE_STRIP);
   for(int i=0, n1=0;i<=keel2;i++){
      if(i>=keel1){
         p = &partsKite[n1].pos;
         glVertex3d(p->x, p->y, p->z);

         if(i<keel2){
            p = &partsKeel[nbPartsKiteKeel - resKeel + i - keel1].pos;
            glVertex3d(p->x, p->y, p->z);
         }
      }
      n1 += (resolution-i+2);
   }
   glEnd();

   // Draw rope
   glColor3f(1, 1, 1);
   glBegin(GL_LINE_STRIP);
   for(int i=0;i<=resRope;i++){
      p = &partsRope[i].pos;
      glVertex3d(p->x, p->y, p->z);
   }
   p = &partsKeel[0].pos;
   //std::cout << "partsKeel[0].pos=" << partsKeel[0].pos << std::endl;
   glVertex3d(p->x, p->y, p->z);
   glEnd();

   // Draw haut et bas
   glColor3f(0.2, 0.2, 0.8);
   glBegin(GL_LINES);
   const Point* ph = &haut.pos;
   const Point* pb = &bas.pos;
   for(int i=0, b=0, c=0;i<=resolution;i++){
      Point* p = &partsKite[i].pos;
      glVertex3d(p->x, p->y, p->z);
      glVertex3d(ph->x, ph->y, ph->z);
      glVertex3d(p->x, p->y, p->z);
      glVertex3d(pb->x, pb->y, pb->z);

      if(i>0){
         p = &partsKite[c].pos;
         glVertex3d(p->x, p->y, p->z);
         glVertex3d(ph->x, ph->y, ph->z);
         glVertex3d(p->x, p->y, p->z);
         glVertex3d(pb->x, pb->y, pb->z);

         if(i<=resolution/2){
            p = &partsKite[b].pos;
            glVertex3d(p->x, p->y, p->z);
            glVertex3d(ph->x, ph->y, ph->z);
            glVertex3d(p->x, p->y, p->z);
            glVertex3d(pb->x, pb->y, pb->z);
         }
      }
      b += resolution - i + 2;
      c += resolution - i + 1;
   }


   glEnd();
  glPopMatrix();
}

