/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

*/

#include "parachute.h"
#include <assert.h>
#include <iostream>
#include <math.h>
#include <GL/gl.h>
#include <GL/glut.h>

#define PI           3.14159265358979323846

CircularParachute::CircularParachute(const Point& centre, int nblevel, double rayon, double massperson, double mass)
: nbLevel(nblevel)
{
  nbParticules = 0; // = (2^nbLevel - 1) / (2 - 1) // srie gomtrique...
  for(int i=0, p=1;i<nbLevel;i++){
     nbParticules += 3 * p;
     p *= 2;
  }
  particules = new Particule[nbParticules];

  int n=0;
  for(int l=1,k=3;l<=nbLevel;l++,k*=2){
     for(int i=0;i<k;i++){
        double angle = 2 * PI * i / k;
        double x = cos(angle) * rayon * l / nbLevel;
        double y = sin(angle) * rayon * l / nbLevel;
        particules[n].pos = centre + Vector(x, y, 0);
        particules[n].mass = mass / nbParticules;
        n++;
     }
  }
  std::cerr << "n=" << n << "  nbParticules=" << nbParticules << std::endl;
  assert(n==nbParticules);

  person.mass = massperson;
  person.pos = centre - Vector(0, 0, 1.5);

  normals = new Vector[nbParticules];
}

CircularParachute::~CircularParachute(){
  delete[] particules;
}

void CircularParachute::registerInSimulator(PhysicSimulatorVerlet* sim){

  // Particules
  sim->addParticule(&person);
  for(int i=0;i<nbParticules;i++)
     sim->addParticule(particules + i);

  // Sticks
  for(int l=1,k=3,debut=0;l<=nbLevel;l++,k*=2){
     for(int i=0;i<k;i++){
        sim->addStick(Stick(particules + debut + i, particules + debut + (i+1)%k));
        if(l==nbLevel){
           if(i%3==0)  // fixer the person under the parachute
              sim->addStick(Stick(particules+debut+i, &person));
            // contrainte de rigidit pour la dernis anneaux
           sim->addStick(Stick(particules+debut+i, particules+debut+(i+k/3)%k));
        }
     }
     debut += k;
  }

  for(int l=1,k=3,debut=0;l<nbLevel;l++,k*=2){
     for(int i=0;i<k;i++){
        sim->addStick(Stick(particules + debut + i, particules + debut + k + 2*i));
        sim->addStick(Stick(particules + debut + i, particules + debut + k + 2*i+1));
        sim->addStick(Stick(particules + debut + i, particules + debut + k + (2*i+2*k-1)%(2*k) ));
     }
     debut += k;
  }

  //Surfaces
  sim->addSurface(SurfaceTriangle(particules+0, particules+1, particules+2));
  for(int l=1,k=3,debut=0;l<nbLevel;l++,k*=2){
     for(int i=0;i<k;i++){
        Particule* p1 = particules + debut + i;
        Particule* p2 = particules + debut + k + 2*i;
        Particule* p3 = particules + debut + k + 2*i+1;
        Particule* p4 = particules + debut + (i+1)%k;
        Particule* p5 = particules + debut + k + (2*i+2)%(2*k);
        sim->addSurface(SurfaceTriangle(p1, p2, p3));
        sim->addSurface(SurfaceTriangle(p1, p3, p4));
        sim->addSurface(SurfaceTriangle(p4, p3, p5));
     }
     debut += k;
  }


}

void CircularParachute::renderGL() const{
  // Resets normals...
  for(int i=0;i<nbParticules;i++)
      normals[i].setZero();

  glColor3ub(200, 33, 33);

  {
     Vector vn = (particules[0].pos - particules[1].pos) ^ (particules[2].pos - particules[0].pos);
     vn.normalize();
     normals[0] -= vn;
     normals[1] -= vn;
     normals[2] -= vn;
  }

  for(int l=1,k=3,debut=0;l<nbLevel;l++,k*=2){
     for(int i=0;i<k;i++){

        int i1 = debut + i;
        int i2 = debut + k + 2*i;
        int i3 = debut + k + 2*i+1;
        int i4 = debut + (i+1)%k;
        int i5 = debut + k + (2*i+2)%(2*k);

        const Point& p1 = particules[i1].pos;
        const Point& p2 = particules[i2].pos;
        const Point& p3 = particules[i3].pos;
        const Point& p4 = particules[i4].pos;
        const Point& p5 = particules[i5].pos;

        Vector v12 = p2 - p1;
        Vector v13 = p3 - p1;
        Vector v34 = p4 - p3;
        Vector v45 = p5 - p4;

        Vector vn = v13 ^ v12;
        vn.normalize();
        normals[i1] += vn;
        normals[i2] += vn;
        normals[i3] += vn;

        vn = v34 ^ v13;
        vn.normalize();
        normals[i1] += vn;
        normals[i3] += vn;
        normals[i4] += vn;

        vn = v34 ^ v45;
        vn.normalize();
        normals[i3] += vn;
        normals[i4] += vn;
        normals[i5] += vn;
     }
     debut += k;
  }

  for(int i=0;i<nbParticules;i++)
     normals[i].normalize();

  {
      glBegin(GL_TRIANGLE_STRIP);
      glNormal3d(0, 0, 1);
      //glNormal3dv(&normals[0].x);
      glVertex3dv(&particules[0].pos.x);
      //glNormal3dv(&normals[1].x);
      glVertex3dv(&particules[1].pos.x);
      //glNormal3dv(&normals[2].x);
      glVertex3dv(&particules[2].pos.x);
      glEnd();
  }

  for(int l=1,k=3,debut=0;l<=nbLevel;l++,k*=2){
     if(l<nbLevel){
        for(int i=0;i<k;i++){
            glBegin(GL_TRIANGLE_STRIP);
            int i1 = debut + i;
            int i2 = debut + k + 2*i;
            int i3 = debut + k + 2*i+1;
            int i4 = debut + (i+1)%k;
            int i5 = debut + k + (2*i+2)%(2*k);

            const Point& p1 = particules[i1].pos;
            const Point& p2 = particules[i2].pos;
            const Point& p3 = particules[i3].pos;
            const Point& p4 = particules[i4].pos;
            const Point& p5 = particules[i5].pos;

            glNormal3dv(&normals[i1].x);
            glVertex3dv(&p2.x);
            glNormal3dv(&normals[i2].x);
            glVertex3dv(&p1.x);
            glNormal3dv(&normals[i3].x);
            glVertex3dv(&p3.x);
            glNormal3dv(&normals[i4].x);
            glVertex3dv(&p4.x);
            glNormal3dv(&normals[i5].x);
            glVertex3dv(&p5.x);

            glEnd();
         }
     }else{
        glColor3ub(180, 180, 180);
        for(int i=0;i<k;i+=3){
            glBegin(GL_LINES);
            glVertex3dv(&particules[debut+i].pos.x);
            glVertex3dv(&person.pos.x);
            glEnd();
         }
     }
     debut += k;
  }

  glColor3ub(30, 30, 190);
  glPushMatrix();
     const Point& p = person.pos;
     glTranslated(p.x, p.y, p.z);
     glutSolidSphere(0.3, 8, 8);
  glPopMatrix();
}
