/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

   Note: Terrain (Height Field)

*/

#if !defined(__TERRAIN_H__)
#define __TERRAIN_H__

#include "geometrie.h"
class Terrain;
#include "object.h"


class Terrain : public Object{
   public:
       Terrain(int w, int h, double ox=0, double oy=0, double resolution=1, double niveau = 0);
       virtual ~Terrain();

       // Par case
       double&    eau(int x, int y) { return ep_eau[x+y*width]; }
       const double&    eau(int x, int y) const { return ep_eau[x+y*width]; }

       double&    sol(int x, int y) { return nivsol[x+y*width]; }
       const double&    sol(int x, int y) const { return nivsol[x+y*width]; }

       Vector&    getVN(int x, int y) { return normals[x+y*width]; }
       const Vector&    getVN(int x, int y) const { return normals[x+y*width]; }

       double     total(int x, int y) const { int i=x+y*width; return nivsol[i] + ep_eau[i];}

       int        getWidth() const { return width;}
       int        getHeight() const { return height;}
       double     getResolution() const { return resolution;}

       void       getMinMax(double& min, double& max) const;

       void       lisser();

       void       approcherVoisin(int x, int y, double cible, double maxdiff);

       void       renderGL() const;

       void       computeVNs();

       // En coordonne mtrique

       double     getLevel(const double x, const double y) const;

       bool       testCollision(const Point& oldpos, Point& newpos);

       static Terrain* generateSinSin(int w, int h, double amplitude = 1);
       static Terrain* generateRandom(int w, int h, int nbmont, int nblac);

       static Terrain* loadFromJPEG(const char* filename, double resXY=1, double resZ=1);

   protected:
       double*  nivsol;  // niveau du sol
       double*  ep_eau;  // paisseur d'eau sur le sol
       Vector*  normals; // vecteurs normals
       int      width;
       int      height;

       double   resolution; // dimension d'une case
       double   ox, oy;  // origine

       double   minTerrain, maxTerrain;

};

class Terrain2 : public Object{
  public:
    void renderGL() const;
};


#endif
