/*
   Projet IFT763 : Objets volant avec Verlet / Contraintes

   (C) Eric Beaudry 2004.
   Eric.Beaudry@USherbrooke.ca
   http://planiart.usherbrooke.ca/~eric/

   Universit de Sherbrooke
   Facult des Sciences
   Dpartement d'Informatique

   Description:
     Timer : Pour synchroniser le temps de simulation avec le temps rel...

   Notes:
     Sous Windows, GetTickCount() n'est mis  jour qu'aux 10ms. Si une meilleur prcision est
     ncessaire, utiliser QueryPerformanceCounter() et QueryPerformanceFrequency().

     Voir rf:
       http://www-106.ibm.com/developerworks/library/l-rt1/
*/


#include "timer.h"

#ifdef _WIN32
 #include <windows.h>

#else
 #include <time.h>
 #include <sys/time.h>
 #include <unistd.h>
 static struct timeval atimeval;
 static struct timezone tz;
#endif



Timer::Timer(int iterPerSec_, int maxStepIter_, double factor_)
 : iterPerSec(iterPerSec_), maxStepIter(maxStepIter_), factor(factor_)
{
   reset();
}

void Timer::reset(){
   currentStep = 0;
#ifdef _WIN32
   startTime = GetTickCount();
#else
   gettimeofday(&atimeval, &tz);
   startTime = atimeval.tv_sec + atimeval.tv_usec/1000;
#endif
}

int Timer::getNbStepToDo(){
   long current;
#ifdef _WIN32
   current = GetTickCount();
#else
   gettimeofday(&atimeval, &tz);
   current = atimeval.tv_sec + atimeval.tv_usec/1000;
#endif

   int nb = (int) ((current-startTime) * iterPerSec / (factor * 1000));
   nb -= currentStep;
   if(nb > maxStepIter){
      startTime += (int) ((nb - maxStepIter) * (factor * 1000) / iterPerSec);
      nb = maxStepIter;
   }
   currentStep += nb;
   return nb;
}

void Timer::waitUnitNextStep(){
   long current;
#ifdef _WIN32
   current = GetTickCount();
#else
   gettimeofday(&atimeval, &tz);
   current = atimeval.tv_sec + atimeval.tv_usec/1000;
#endif

   int nb = (int) ((current-startTime) * iterPerSec / 1000);
   if(nb>0) return;
   long mod = ((current-startTime) * iterPerSec) % 1000;
#ifdef _WIN32
   Sleep(mod);
#else
   struct timespec timeOut,remains;
   timeOut.tv_sec = 0;
   timeOut.tv_nsec = mod * 1000000;
   nanosleep(&timeOut, &remains);
#endif

}
